#undef  FASTSEARCH
#undef  STRINGLIB
#undef  STRINGLIB_SIZEOF_CHAR
#undef  STRINGLIB_MAX_CHAR
#undef  STRINGLIB_CHAR
#undef  STRINGLIB_STR
#undef  STRINGLIB_LEN
#undef  STRINGLIB_NEW
#undef STRINGLIB_IS_UNICODE

